"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = loginUser;
const api_login_1 = __importDefault(require("../../functions/api/users/api-login"));
const debug_log_1 = __importDefault(require("../../utils/logging/debug-log"));
const grab_cookie_expirt_date_1 = __importDefault(require("../../utils/grab-cookie-expirt-date"));
const grab_api_path_1 = __importDefault(require("../../utils/backend/users/grab-api-path"));
const query_dsql_api_1 = __importDefault(require("../../functions/api/query-dsql-api"));
const post_login_response_handler_1 = __importDefault(require("../../functions/backend/auth/post-login-response-handler"));
function debugFn(log, label) {
    (0, debug_log_1.default)({ log, addTime: true, title: "loginUser", label });
}
/**
 * # Login A user
 */
function loginUser(_a) {
    return __awaiter(this, arguments, void 0, function* ({ apiKey, payload, database, additionalFields, response, encryptionKey, encryptionSalt, email_login, email_login_code, temp_code_field, token, skipPassword, apiUserID, skipWriteAuthFile, dbUserId, debug, cleanupTokens, secureCookie, useLocal, apiVersion = "v1", }) {
        const COOKIE_EXPIRY_DATE = (0, grab_cookie_expirt_date_1.default)();
        const defaultTempLoginFieldName = "temp_login_code";
        const emailLoginTempCodeFieldName = email_login
            ? temp_code_field
                ? temp_code_field
                : defaultTempLoginFieldName
            : undefined;
        const finalEncryptionKey = encryptionKey || process.env.DSQL_ENCRYPTION_PASSWORD;
        const finalEncryptionSalt = encryptionSalt || process.env.DSQL_ENCRYPTION_SALT;
        if (!(finalEncryptionKey === null || finalEncryptionKey === void 0 ? void 0 : finalEncryptionKey.match(/.{8,}/))) {
            console.log("Encryption key is invalid");
            return {
                success: false,
                payload: null,
                msg: "Encryption key is invalid",
            };
        }
        if (!(finalEncryptionSalt === null || finalEncryptionSalt === void 0 ? void 0 : finalEncryptionSalt.match(/.{8,}/))) {
            console.log("Encryption salt is invalid");
            return {
                success: false,
                payload: null,
                msg: "Encryption salt is invalid",
            };
        }
        /**
         * Initialize HTTP response variable
         */
        let httpResponse = {
            success: false,
        };
        const apiLoginParams = {
            database,
            email: payload.email,
            username: payload.username,
            password: payload.password,
            skipPassword,
            encryptionKey: finalEncryptionKey,
            additionalFields,
            email_login,
            email_login_code,
            email_login_field: emailLoginTempCodeFieldName,
            token,
            dbUserId,
            debug,
        };
        /**
         * Check for local DB settings
         *
         * @description Look for local db settings in `.env` file and by pass the http request if available
         */
        if (useLocal) {
            httpResponse = yield (0, api_login_1.default)(apiLoginParams);
        }
        else {
            httpResponse = yield (0, query_dsql_api_1.default)({
                path: (0, grab_api_path_1.default)({
                    paradigm: "auth",
                    action: "login",
                    database,
                    apiVersion,
                }),
                apiKey,
                body: apiLoginParams,
                method: "POST",
            });
        }
        if (debug) {
            debugFn(httpResponse, "httpResponse");
        }
        /**
         * # Send Response
         */
        if (httpResponse === null || httpResponse === void 0 ? void 0 : httpResponse.success) {
            (0, post_login_response_handler_1.default)({
                database,
                httpResponse,
                cleanupTokens,
                debug,
                encryptionKey,
                encryptionSalt,
                response,
                secureCookie,
                skipWriteAuthFile,
                token,
            });
        }
        return httpResponse;
    });
}
