"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = googleAuth;
const api_google_login_1 = __importDefault(require("../../../functions/api/users/social/api-google-login"));
const query_dsql_api_1 = __importDefault(require("../../../functions/api/query-dsql-api"));
const grab_api_path_1 = __importDefault(require("../../../utils/backend/users/grab-api-path"));
const post_login_response_handler_1 = __importDefault(require("../../../functions/backend/auth/post-login-response-handler"));
/**
 * # SERVER FUNCTION: Login with google Function
 */
function googleAuth(_a) {
    return __awaiter(this, arguments, void 0, function* ({ apiKey, token, database, response, encryptionKey, encryptionSalt, additionalFields, additionalData, debug, secureCookie, loginOnly, useLocal, apiVersion, skipWriteAuthFile, cleanupTokens, }) {
        const finalEncryptionKey = encryptionKey || process.env.DSQL_ENCRYPTION_PASSWORD;
        const finalEncryptionSalt = encryptionSalt || process.env.DSQL_ENCRYPTION_SALT;
        if (!(finalEncryptionKey === null || finalEncryptionKey === void 0 ? void 0 : finalEncryptionKey.match(/.{8,}/))) {
            console.log("Encryption key is invalid");
            return {
                success: false,
                payload: null,
                msg: "Encryption key is invalid",
            };
        }
        if (!(finalEncryptionSalt === null || finalEncryptionSalt === void 0 ? void 0 : finalEncryptionSalt.match(/.{8,}/))) {
            console.log("Encryption salt is invalid");
            return {
                success: false,
                payload: null,
                msg: "Encryption salt is invalid",
            };
        }
        /**
         * Check inputs
         *
         * @description Check inputs
         */
        if (!token || (token === null || token === void 0 ? void 0 : token.match(/ /))) {
            return {
                success: false,
                payload: null,
                msg: "Please enter Google Access Token",
            };
        }
        /**
         * Initialize HTTP response variable
         */
        let httpResponse = {
            success: false,
        };
        const googleAuthParams = {
            token,
            additionalFields,
            additionalData,
            debug,
            loginOnly,
            database,
        };
        if (useLocal) {
            if (debug) {
                console.log(`Google login with Local Paradigm ...`);
            }
            httpResponse = yield (0, api_google_login_1.default)(googleAuthParams);
        }
        else {
            httpResponse = yield (0, query_dsql_api_1.default)({
                path: (0, grab_api_path_1.default)({
                    paradigm: "auth",
                    action: "google-login",
                    database,
                    apiVersion,
                }),
                apiKey,
                body: googleAuthParams,
                method: "POST",
            });
        }
        /**
         * Make https request
         *
         * @description make a request to datasquirel.com
         */
        if ((httpResponse === null || httpResponse === void 0 ? void 0 : httpResponse.success) && (httpResponse === null || httpResponse === void 0 ? void 0 : httpResponse.payload) && database) {
            (0, post_login_response_handler_1.default)({
                database,
                httpResponse,
                cleanupTokens,
                debug,
                encryptionKey,
                encryptionSalt,
                response,
                secureCookie,
                skipWriteAuthFile,
            });
        }
        return httpResponse;
    });
}
